@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2021, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2018, 2020 - 2024, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,20 +20,19 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// https://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <Table.h>
 #include <Duration.h>
-#include <format.h>
+#include <GapsTable.h>
 #include <commands.h>
-#include <timew.h>
 #include <iostream>
+#include <timew.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdGaps (
-  const CLI& cli,
+  CLI& cli,
   Rules& rules,
   Database& database)
 {
@@ -45,82 +44,41 @@ int CmdGaps (
   Range default_range = {};
   expandIntervalHint (":" + report_hint, default_range);
 
-  auto filter = cli.getFilter (default_range);
+  auto range = cli.getRange (default_range);
+  auto tags = cli.getTags ();
+  auto filter = Interval {range, tags};
 
   // Is the :blank hint being used?
   bool blank = cli.getHint ("blank", false);
 
   std::vector <Range> untracked;
   if (blank)
-    untracked = subtractRanges ({filter}, getAllExclusions (rules, filter));
-  else
-    untracked = getUntracked (database, rules, filter);
-
-  Table table;
-  table.width (1024);
-  table.colorHeader (Color ("underline"));
-  table.add ("Wk");
-  table.add ("Date");
-  table.add ("Day");
-  table.add ("Start", false);
-  table.add ("End", false);
-  table.add ("Time", false);
-  table.add ("Total", false);
-
-  // Each day is rendered separately.
-  time_t grand_total = 0;
-  Datetime previous;
-  for (Datetime day = filter.start; day < filter.end; day++)
   {
-    auto day_range = getFullDay (day);
-    time_t daily_total = 0;
-
-    int row = -1;
-    for (auto& gap : subset (day_range, untracked))
-    {
-      row = table.addRow ();
-
-      if (day != previous)
-      {
-        table.set (row, 0, format ("W{1}", day.week ()));
-        table.set (row, 1, day.toString ("Y-M-D"));
-        table.set (row, 2, day.dayNameShort (day.dayOfWeek ()));
-        previous = day;
-      }
-
-      // Intersect track with day.
-      auto today = day_range.intersect (gap);
-      if (gap.is_open ())
-        today.end = Datetime ();
-
-      table.set (row, 3, today.start.toString ("h:N:S"));
-      table.set (row, 4, (gap.is_open () ? "-" : today.end.toString ("h:N:S")));
-      table.set (row, 5, Duration (today.total ()).formatHours ());
-
-      daily_total += today.total ();
-    }
-
-    if (row != -1)
-      table.set (row, 6, Duration (daily_total).formatHours ());
-
-    grand_total += daily_total;
+    untracked = subtractRanges ({range}, getAllExclusions (rules, range));
+  }
+  else
+  {
+    untracked = getUntracked (database, rules, filter);
   }
 
-  // Add the total.
-  table.set (table.addRow (), 6, " ", Color ("underline"));
-  table.set (table.addRow (), 6, Duration (grand_total).formatHours ());
-
-  if (table.rows () > 2)
+  if (untracked.empty ())
   {
+    if (verbose)
+    {
+      std::cout << "No gaps found.\n";
+    }
+  }
+  else
+  {
+    auto table = GapsTable::builder ()
+      .withRange (range)
+      .withIntervals (untracked)
+      .build ();
+
     std::cout << '\n'
               << table.render ()
               << '\n';
   }
-  else
-  {
-    if (verbose)
-      std::cout << "No gaps found.\n";
-  }
 
   return 0;
 }
