@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2020, 2022 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, 2022 - 2024, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// https://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -52,9 +52,11 @@ int CmdHelpUsage (const Extensions& extensions)
             << "       timew join @<id> @<id>\n"
             << "       timew lengthen @<id> [@<id> ...] <duration>\n"
             << "       timew modify (start|end) @<id> <date>\n"
+            << "       timew modify range @<id> <interval>\n"
             << "       timew month [<interval>] [<tag> ...]\n"
             << "       timew move @<id> <date>\n"
             << "       timew [report] <report> [<interval>] [<tag> ...]\n"
+            << "       timew retag @<id> [@<id> ...] <tag> [<tag> ...]\n"
             << "       timew shorten @<id> [@<id> ...] <duration>\n"
             << "       timew show\n"
             << "       timew split @<id> [@<id> ...]\n"
@@ -112,7 +114,7 @@ int CmdHelpUsage (const Extensions& extensions)
 }
 
 int CmdHelp (
-  const CLI& cli,
+  CLI& cli,
   const Extensions& extensions)
 {
   auto words = cli.getWords ();
