@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2018, 2020 - 2025, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// https://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -33,12 +33,13 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdShorten (
-  const CLI& cli,
+  CLI& cli,
   Rules& rules,
   Database& database,
   Journal& journal)
 {
   const bool verbose = rules.getBoolean ("verbose");
+  const bool do_adjust = cli.getHint ("adjust", false);
 
   std::set <int> ids = cli.getIds ();
 
@@ -94,7 +95,7 @@ int CmdShorten (
     database.deleteInterval (interval);
 
     interval.end -= dur.toTime_t ();
-    validate (cli, rules, database, interval);
+    autoAdjust (do_adjust, rules, database, interval);
     database.addInterval (interval, verbose);
 
     if (verbose)
