@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2020, 2022 - 2025, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// https://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -30,12 +30,14 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdStart (
-  const CLI& cli,
+  CLI& cli,
   Rules& rules,
   Database& database,
   Journal& journal)
 {
   const bool verbose = rules.getBoolean ("verbose");
+  const bool do_fill = cli.getHint ("fill", false);
+  const bool do_adjust = cli.getHint ("adjust", false);
   const Datetime now {};
 
   auto range = cli.getRange ({now, 0});
@@ -61,7 +63,12 @@ int CmdStart (
   journal.startTransaction ();
   auto interval = Interval {range, tags};
 
-  if (validate (cli, rules, database, interval))
+  if (do_fill)
+  {
+    fillRangeStart (rules, database, interval);
+  }
+  
+  if (autoAdjust (do_adjust, rules, database, interval))
   {
     database.addInterval (interval, verbose);
     journal.endTransaction ();
