@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2016 - 2023, Thomas Lauf, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2018, 2020, 2022 - 2025, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// https://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -30,12 +30,14 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 int CmdTrack (
-  const CLI& cli,
+  CLI& cli,
   Rules& rules,
   Database& database,
   Journal& journal)
 {
   const bool verbose = rules.getBoolean ("verbose");
+  const bool do_fill = cli.getHint ("fill", false);
+  const bool do_adjust = cli.getHint ("adjust", false);
 
   // We expect no ids
   if (! cli.getIds ().empty ())
@@ -60,7 +62,11 @@ int CmdTrack (
   auto filter = Interval {range, tags};
 
   // Validation must occur before flattening.
-  validate (cli, rules, database, filter);
+  if (do_fill)
+  {
+    fillRange (rules, database, filter);
+  }
+  autoAdjust (do_adjust, rules, database, filter);
 
   for (auto& interval : flatten (filter, getAllExclusions (rules, range)))
   {
