@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2015 - 2021, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2017, 2019 - 2021, 2023, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,19 +20,18 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <cmake.h>
 #include <Pig.h>
-#include <shared.h>
-#include <unicode.h>
-#include <utf8.h>
 #include <algorithm>
-#include <sstream>
 #include <cinttypes>
 #include <cstdlib>
+#include <shared.h>
+#include <sstream>
+#include <unicode.h>
+#include <utf8.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 Pig::Pig (const std::string& text)
@@ -103,13 +102,15 @@ bool Pig::skipLiteral (const std::string& literal)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Pig::skipPartial (const std::string& reference, std::string& result)
+bool Pig::skipPartial (const std::string& reference, std::string& result,
+                       bool ignore_case)
 {
   // Walk the common substring.
   auto pos = 0;
   while (reference[pos] &&
          (*_text)[_cursor + pos] &&
-         reference[pos] == (*_text)[_cursor + pos])
+         ((reference[pos] == (*_text)[_cursor + pos] && !ignore_case) ||
+          (reference[pos] == tolower((*_text)[_cursor + pos]) && ignore_case)))
     ++pos;
 
   if (pos > 0)
@@ -122,6 +123,28 @@ bool Pig::skipPartial (const std::string& reference, std::string& result)
   return false;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// ASCII-only version of getUntil. Has better performance characteristics at
+// the cost of not considering utf-8 characters correctly. Use with caution.
+bool Pig::getUntilAscii (char end, std::string& result)
+{
+  auto save = _cursor;
+  auto found = _text->find (end, _cursor + 1);
+
+  if (found == std::string::npos)
+  {
+    found = _text->size ();
+    result = _text->substr (_cursor, found - _cursor);
+    _cursor = found;
+    return true;
+  }
+
+  result = _text->substr (_cursor, found - _cursor);
+  _cursor = _cursor + result.size();
+
+  return _cursor > save;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool Pig::getUntil (int end, std::string& result)
 {
@@ -217,7 +240,7 @@ bool Pig::getDigit2 (int& result)
   {
     if (unicodeLatinDigit ((*_text)[_cursor + 1]))
     {
-      result = strtoimax (_text->substr (_cursor, 2).c_str (), NULL, 10);
+      result = strtoimax (_text->substr (_cursor, 2).c_str (), nullptr, 10);
       _cursor += 2;
       return true;
     }
@@ -235,7 +258,7 @@ bool Pig::getDigit3 (int& result)
     {
       if (unicodeLatinDigit ((*_text)[_cursor + 2]))
       {
-        result = strtoimax (_text->substr (_cursor, 3).c_str (), NULL, 10);
+        result = strtoimax (_text->substr (_cursor, 3).c_str (), nullptr, 10);
         _cursor += 3;
         return true;
       }
@@ -256,7 +279,7 @@ bool Pig::getDigit4 (int& result)
       {
         if (unicodeLatinDigit ((*_text)[_cursor + 3]))
         {
-          result = strtoimax (_text->substr (_cursor, 4).c_str (), NULL, 10);
+          result = strtoimax (_text->substr (_cursor, 4).c_str (), nullptr, 10);
           _cursor += 4;
           return true;
         }
@@ -300,7 +323,7 @@ bool Pig::getDigits (long long& result)
 
   if (_cursor > save)
   {
-    result = strtoimax (_text->substr (save, _cursor - save).c_str (), NULL, 10);
+    result = strtoimax (_text->substr (save, _cursor - save).c_str (), nullptr, 10);
     return true;
   }
 
@@ -422,7 +445,7 @@ bool Pig::getNumber (double& result)
   std::string s;
   if (getNumber (s))
   {
-    result = std::strtod (s.c_str (), NULL);
+    result = std::strtod (s.c_str (), nullptr);
     return true;
   }
 
@@ -472,7 +495,7 @@ bool Pig::getDecimal (double& result)
   std::string s;
   if (getDecimal (s))
   {
-    result = std::strtod (s.c_str (), NULL);
+    result = std::strtod (s.c_str (), nullptr);
     return true;
   }
 
@@ -497,7 +520,7 @@ bool Pig::getQuoted (int quote, std::string& result)
   {
     i = _text->find (quote, i);
     if (i == std::string::npos)
-      return false;  // Unclosed quote.  Short cut, not definitive.
+      return false;  // Unclosed quote. Shortcut, not definitive.
 
     if (i == start)
     {
@@ -517,7 +540,7 @@ bool Pig::getQuoted (int quote, std::string& result)
       while (j >= start && (*_text)[j] == '\\')
       {
         // Toggle flag for each further backslash encountered.
-        is_escaped_quote = is_escaped_quote ? false : true;
+        is_escaped_quote = !is_escaped_quote;
         --j;
       }
 
