@@ -20,14 +20,14 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
 #ifndef INCLUDED_RX
 #define INCLUDED_RX
 
-#include <regex.h>
+#include <regex>
 #include <string>
 #include <vector>
 
@@ -51,7 +51,7 @@ private:
   bool _compiled       {false};
   std::string _pattern {};
   bool _case_sensitive {false};
-  regex_t _regex;
+  std::regex _regex;
 };
 
 #endif
