@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2017, 2019 - 2021, 2023, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,16 +20,20 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <cmake.h>
 #include <Table.h>
-#include <shared.h>
 #include <format.h>
-#include <utf8.h>
+#include <shared.h>
+#ifndef _WIN32
 #include <unistd.h>
+#endif
+#ifdef _WIN32
+#include <io.h>
+#endif
+#include <utf8.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 void Table::add (const std::string& col, bool alignLeft, bool wrap)
@@ -42,17 +46,17 @@ void Table::add (const std::string& col, bool alignLeft, bool wrap)
 ////////////////////////////////////////////////////////////////////////////////
 int Table::addRow ()
 {
-  _data.push_back (std::vector <std::string> (_columns.size (), ""));
-  _color.push_back (std::vector <Color> (_columns.size (), Color::nocolor));
-  _oddness.push_back (_data.size () % 2 ? true : false);
+  _data.emplace_back (_columns.size (), "");
+  _color.emplace_back (_columns.size (), Color::nocolor);
+  _oddness.push_back (_data.size () % 2 != 0);
   return _data.size () - 1;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Table::addRowOdd ()
 {
-  _data.push_back (std::vector <std::string> (_columns.size (), ""));
-  _color.push_back (std::vector <Color> (_columns.size (), Color::nocolor));
+  _data.emplace_back (_columns.size (), "");
+  _color.emplace_back (_columns.size (), Color::nocolor);
   _oddness.push_back (true);
   return _data.size () - 1;
 }
@@ -60,8 +64,8 @@ int Table::addRowOdd ()
 ////////////////////////////////////////////////////////////////////////////////
 int Table::addRowEven ()
 {
-  _data.push_back (std::vector <std::string> (_columns.size (), ""));
-  _color.push_back (std::vector <Color> (_columns.size (), Color::nocolor));
+  _data.emplace_back (_columns.size (), "");
+  _color.emplace_back (_columns.size (), Color::nocolor);
   _oddness.push_back (false);
   return _data.size () - 1;
 }
@@ -97,7 +101,12 @@ std::string Table::render ()
 {
   // Piped output disables color, unless overridden.
   if (! _forceColor &&
-      ! isatty (STDOUT_FILENO))
+#ifdef _WIN32
+      ! _isatty(_fileno(stdout))
+#else
+      ! isatty (STDOUT_FILENO)
+#endif
+      )
   {
     _header     = Color ("");
     _odd        = Color ("");
@@ -183,7 +192,7 @@ std::string Table::render ()
   std::vector <std::vector <std::string>> headers;
   for (unsigned int c = 0; c < _columns.size (); ++c)
   {
-    headers.push_back ({});
+    headers.emplace_back ();
     renderCell (headers[c], _columns[c], widths[c], _align[c], _wrap[c], _header);
 
     if (headers[c].size () > max_lines)
@@ -222,7 +231,7 @@ std::string Table::render ()
     out += extra;
 
     // Trim right.
-    out.erase (out.find_last_not_of (" ") + 1);
+    out.erase (out.find_last_not_of (' ') + 1);
     out += '\n';
 
     // Stop if the line limit is exceeded.
@@ -266,7 +275,7 @@ std::string Table::render ()
       cell_color = row_color;
       cell_color.blend (_color[row][col]);
 
-      cells.push_back (std::vector <std::string> ());
+      cells.emplace_back ();
       renderCell (cells[col], _data[row][col], widths[col], _align[col], _wrap[col], cell_color);
 
       if (cells[col].size () > max_lines)
@@ -304,7 +313,7 @@ std::string Table::render ()
       out += (oddness ? extra_odd : extra_even);
 
       // Trim right.
-      out.erase (out.find_last_not_of (" ") + 1);
+      out.erase (out.find_last_not_of (' ') + 1);
       out += '\n';
 
       // Stop if the line limit is exceeded.
