@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2010 - 2021, Paul Beckingham, Federico Hernandez.
+// Copyright 2016 - 2017, 2019 - 2021, 2023, Gothenburg Bit Factory.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,17 +20,15 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <cmake.h>
 #include <Tree.h>
 #include <algorithm>
-#include <iostream>
-#include <sstream>
-#include <shared.h>
 #include <format.h>
+#include <shared.h>
+#include <sstream>
 
 ////////////////////////////////////////////////////////////////////////////////
 //  - Tree, Branch and Node are synonymous.
@@ -39,7 +37,7 @@
 //  - The destructor will delete all branches recursively.
 //  - Tree::enumerate is a snapshot, and is invalidated by modification.
 //  - Branch sequence is preserved.
-void Tree::addBranch (std::shared_ptr <Tree> branch)
+void Tree::addBranch (const std::shared_ptr <Tree>& branch)
 {
   if (! branch)
     throw "Failed to allocate memory for parse tree.";
@@ -48,7 +46,7 @@ void Tree::addBranch (std::shared_ptr <Tree> branch)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Tree::removeBranch (std::shared_ptr <Tree> branch)
+void Tree::removeBranch (const std::shared_ptr <Tree>& branch)
 {
   for (auto i = _branches.begin (); i != _branches.end (); ++i)
   {
@@ -67,7 +65,7 @@ void Tree::removeAllBranches ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void Tree::replaceBranch (std::shared_ptr <Tree> from, std::shared_ptr <Tree> to)
+void Tree::replaceBranch (const std::shared_ptr <Tree>& from, const std::shared_ptr <Tree>& to)
 {
   for (unsigned int i = 0; i < _branches.size (); ++i)
   {
@@ -213,7 +211,7 @@ std::shared_ptr <Tree> Tree::find (const std::string& path)
 
 ////////////////////////////////////////////////////////////////////////////////
 std::string Tree::dumpNode (
-  const std::shared_ptr <Tree> t,
+  const std::shared_ptr <Tree>& t,
   int depth) const
 {
   std::stringstream out;
@@ -231,26 +229,26 @@ std::string Tree::dumpNode (
   std::string atts;
   for (auto& a : t->_attributes)
   {
-    if (atts != "")
+    if (!atts.empty ())
       atts += ' ';
 
     atts += a.first + "='\033[33m" + a.second + "\033[0m'";
   }
 
-  if (atts.length ())
+  if (!atts.empty ())
     out << ' ' << atts;
 
   // Dump tags.
   std::string tags;
   for (auto& tag : t->_tags)
   {
-    if (tags.length ())
+    if (!tags.empty ())
       tags += ' ';
 
     tags += "\033[32m" + tag + "\033[0m";
   }
 
-  if (tags.length ())
+  if (!tags.empty ())
     out << ' ' << tags;
   out << '\n';
 
