@@ -20,7 +20,7 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://opensource.org/license/mit
 //
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -32,7 +32,9 @@
 #include <format.h>
 #include <iomanip>
 #include <sstream>
+#ifndef _WIN32
 #include <strings.h>
+#endif
 #include <utf8.h>
 
 ////////////////////////////////////////////////////////////////////////////////
